@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) 2010, 2012  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2010, 2012, 2014  Internet Systems Consortium, Inc. ("ISC")
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -14,14 +14,19 @@
 # OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 # PERFORMANCE OF THIS SOFTWARE.
 
-# $Id$
+echo "I:(Native PKCS#11)" >&2
+rsafail=0 eccfail=0
 
-../../../tools/genrandom 400 random.data
+sh ../testcrypto.sh -q rsa || rsafail=1
+sh ../testcrypto.sh -q ecdsa || eccfail=1
 
-if $KEYGEN -q -a RSAMD5 -b 512 -n zone -r random.data foo > /dev/null 2>&1
-then
-    rm -f Kfoo*
+if [ $rsafail = 0 -a $eccfail = 0 ]; then
+	echo both > supported
+elif [ $rsafail = 1 -a $eccfail = 1 ]; then
+	echo "I:This test requires PKCS#11 support for either RSA or ECDSA cryptography." >&2
+	exit 255
+elif [ $rsafail = 0 ]; then
+	echo rsaonly > supported
 else
-    echo "I:This test requires that --with-openssl was used." >&2
-    exit 1
+        echo ecconly > supported
 fi
