@@ -1,6 +1,6 @@
 #!/bin/sh -e
 #
-# Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2012, 2014  Internet Systems Consortium, Inc. ("ISC")
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -17,8 +17,7 @@
 SYSTEMTESTTOP=..
 . $SYSTEMTESTTOP/conf.sh
 
-$SHELL clean.sh 
-
+sh clean.sh 
 test -r $RANDFILE || $GENRANDOM 400 $RANDFILE
 
-(cd zones && $SHELL genzones.sh)
+(cd zones && sh genzones.sh)
