@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2008, 2009  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2006, 2008, 2009, 2011, 2014  Internet Systems Consortium, Inc. ("ISC")
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,7 +14,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: nsec3hash.c,v 1.6 2009-10-06 21:20:44 each Exp $ */
+/* $Id: nsec3hash.c,v 1.8 2011/11/02 23:46:24 tbox Exp $ */
 
 #include <config.h>
 
@@ -59,8 +59,10 @@ check_result(isc_result_t result, const char *message) {
 }
 
 static void
-usage() {
-	fatal("salt hash iterations domain");
+usage(void) {
+	fprintf(stderr, "Usage: %s salt algorithm iterations domain\n",
+		program);
+	exit(1);
 }
 
 int
