@@ -1,5 +1,5 @@
 /*
- * Portions Copyright (C) 2005, 2007, 2009, 2010  Internet Systems Consortium, Inc. ("ISC")
+ * Portions Copyright (C) 2005, 2007, 2009-2013  Internet Systems Consortium, Inc. ("ISC")
  * Portions Copyright (C) 1999-2001  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -50,7 +50,7 @@
  * USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: dlz.c,v 1.10 2010-12-20 23:47:20 tbox Exp $ */
+/* $Id$ */
 
 /*! \file */
 
@@ -60,10 +60,11 @@
 
 #include <config.h>
 
+#include <dns/db.h>
+#include <dns/dlz.h>
 #include <dns/fixedname.h>
 #include <dns/log.h>
 #include <dns/master.h>
-#include <dns/dlz.h>
 #include <dns/ssu.h>
 #include <dns/zone.h>
 
@@ -113,26 +114,41 @@ isc_result_t
 dns_dlzallowzonexfr(dns_view_t *view, dns_name_t *name,
 		    isc_sockaddr_t *clientaddr, dns_db_t **dbp)
 {
-	isc_result_t result;
+	isc_result_t result = ISC_R_NOTFOUND;
 	dns_dlzallowzonexfr_t allowzonexfr;
-	dns_dlzdb_t *dlzdatabase;
+	dns_dlzdb_t *dlzdb;
 
 	/*
 	 * Performs checks to make sure data is as we expect it to be.
 	 */
-	REQUIRE(DNS_DLZ_VALID(view->dlzdatabase));
 	REQUIRE(name != NULL);
 	REQUIRE(dbp != NULL && *dbp == NULL);
 
-	/* ask driver if the zone is supported */
-	dlzdatabase = view->dlzdatabase;
-	allowzonexfr = dlzdatabase->implementation->methods->allowzonexfr;
-	result = (*allowzonexfr)(dlzdatabase->implementation->driverarg,
-				 dlzdatabase->dbdata, dlzdatabase->mctx,
-				 view->rdclass, name, clientaddr, dbp);
+	/*
+	 * Find a driver in which the zone exists and transfer is supported
+	 */
+	for (dlzdb = ISC_LIST_HEAD(view->dlz_searched);
+	     dlzdb != NULL;
+	     dlzdb = ISC_LIST_NEXT(dlzdb, link))
+	{
+		REQUIRE(DNS_DLZ_VALID(dlzdb));
+
+		allowzonexfr = dlzdb->implementation->methods->allowzonexfr;
+		result = (*allowzonexfr)(dlzdb->implementation->driverarg,
+					 dlzdb->dbdata, dlzdb->mctx,
+					 view->rdclass, name, clientaddr, dbp);
+
+		/*
+		 * if ISC_R_NOPERM, we found the right database but
+		 * the zone may not transfer.
+		 */
+		if (result == ISC_R_SUCCESS || result == ISC_R_NOPERM)
+			return (result);
+	}
 
 	if (result == ISC_R_NOTIMPLEMENTED)
-		return (ISC_R_NOTFOUND);
+		result = ISC_R_NOTFOUND;
+
 	return (result);
 }
 
@@ -142,6 +158,7 @@ dns_dlzcreate(isc_mem_t *mctx, const char *dlzname, const char *drivername,
 {
 	dns_dlzimplementation_t *impinfo;
 	isc_result_t result;
+	dns_dlzdb_t *db = NULL;
 
 	/*
 	 * initialize the dlz_implementations list, this is guaranteed
@@ -180,30 +197,34 @@ dns_dlzcreate(isc_mem_t *mctx, const char *dlzname, const char *drivername,
 	}
 
 	/* Allocate memory to hold the DLZ database driver */
-	(*dbp) = isc_mem_get(mctx, sizeof(dns_dlzdb_t));
-	if ((*dbp) == NULL) {
+	db = isc_mem_get(mctx, sizeof(dns_dlzdb_t));
+	if (db == NULL) {
 		RWUNLOCK(&dlz_implock, isc_rwlocktype_read);
 		return (ISC_R_NOMEMORY);
 	}
 
 	/* Make sure memory region is set to all 0's */
-	memset((*dbp), 0, sizeof(dns_dlzdb_t));
+	memset(db, 0, sizeof(dns_dlzdb_t));
 
-	(*dbp)->implementation = impinfo;
+	ISC_LINK_INIT(db, link);
+	db->implementation = impinfo;
+	if (dlzname != NULL)
+		db->dlzname = isc_mem_strdup(mctx, dlzname);
 
 	/* Create a new database using implementation 'drivername'. */
 	result = ((impinfo->methods->create)(mctx, dlzname, argc, argv,
 					     impinfo->driverarg,
-					     &(*dbp)->dbdata));
+					     &db->dbdata));
 
 	/* mark the DLZ driver as valid */
 	if (result == ISC_R_SUCCESS) {
 		RWUNLOCK(&dlz_implock, isc_rwlocktype_read);
-		(*dbp)->magic = DNS_DLZ_MAGIC;
-		isc_mem_attach(mctx, &(*dbp)->mctx);
+		db->magic = DNS_DLZ_MAGIC;
+		isc_mem_attach(mctx, &db->mctx);
 		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
 			      DNS_LOGMODULE_DLZ, ISC_LOG_DEBUG(2),
 			      "DLZ driver loaded successfully.");
+		*dbp = db;
 		return (ISC_R_SUCCESS);
 	} else {
 		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
@@ -213,7 +234,7 @@ dns_dlzcreate(isc_mem_t *mctx, const char *dlzname, const char *drivername,
 
 	/* impinfo->methods->create failed. */
 	RWUNLOCK(&dlz_implock, isc_rwlocktype_read);
-	isc_mem_put(mctx, (*dbp), sizeof(dns_dlzdb_t));
+	isc_mem_put(mctx, db, sizeof(dns_dlzdb_t));
 	return (result);
 }
 
@@ -232,15 +253,15 @@ dns_dlzdestroy(dns_dlzdb_t **dbp) {
 	 */
 	REQUIRE(dbp != NULL && DNS_DLZ_VALID(*dbp));
 
-#ifdef BIND9
 	if ((*dbp)->ssutable != NULL) {
 		dns_ssutable_detach(&(*dbp)->ssutable);
 	}
-#endif
 
 	/* call the drivers destroy method */
 	if ((*dbp) != NULL) {
 		mctx = (*dbp)->mctx;
+		if ((*dbp)->dlzname != NULL)
+			isc_mem_free(mctx, (*dbp)->dlzname);
 		destroy = (*dbp)->implementation->methods->destroy;
 		(*destroy)((*dbp)->implementation->driverarg,(*dbp)->dbdata);
 		/* return memory */
@@ -251,63 +272,6 @@ dns_dlzdestroy(dns_dlzdb_t **dbp) {
 	*dbp = NULL;
 }
 
-
-isc_result_t
-dns_dlzfindzone(dns_view_t *view, dns_name_t *name, unsigned int minlabels,
-		dns_db_t **dbp)
-{
-	dns_fixedname_t fname;
-	dns_name_t *zonename;
-	unsigned int namelabels;
-	unsigned int i;
-	isc_result_t result;
-	dns_dlzfindzone_t findzone;
-	dns_dlzdb_t *dlzdatabase;
-
-	/*
-	 * Performs checks to make sure data is as we expect it to be.
-	 */
-	REQUIRE(DNS_DLZ_VALID(view->dlzdatabase));
-	REQUIRE(name != NULL);
-	REQUIRE(dbp != NULL && *dbp == NULL);
-
-	/* setup a "fixed" dns name */
-	dns_fixedname_init(&fname);
-	zonename = dns_fixedname_name(&fname);
-
-	/* count the number of labels in the name */
-	namelabels = dns_name_countlabels(name);
-
-	/*
-	 * loop through starting with the longest domain name and
-	 * trying shorter names portions of the name until we find a
-	 * match, have an error, or are below the 'minlabels'
-	 * threshold.  minlabels is 0, if the standard database didn't
-	 * have a zone name match.  Otherwise minlabels is the number
-	 * of labels in that name.  We need to beat that for a
-	 * "better" match for the DLZ database to be authoritative
-	 * instead of the standard database.
-	 */
-	for (i = namelabels; i > minlabels && i > 1; i--) {
-		if (i == namelabels) {
-			result = dns_name_copy(name, zonename, NULL);
-			if (result != ISC_R_SUCCESS)
-				return (result);
-		} else
-			dns_name_split(name, i, NULL, zonename);
-
-		/* ask SDLZ driver if the zone is supported */
-		dlzdatabase = view->dlzdatabase;
-		findzone = dlzdatabase->implementation->methods->findzone;
-		result = (*findzone)(dlzdatabase->implementation->driverarg,
-				     dlzdatabase->dbdata, dlzdatabase->mctx,
-				     view->rdclass, zonename, dbp);
-		if (result != ISC_R_NOTFOUND)
-			return (result);
-	}
-	return (ISC_R_NOTFOUND);
-}
-
 /*%
  * Registers a DLZ driver.  This basically just adds the dlz
  * driver to the list of available drivers in the dlz_implementations list.
@@ -517,7 +481,6 @@ dns_dlzunregister(dns_dlzimplementation_t **dlzimp) {
 	RWUNLOCK(&dlz_implock, isc_rwlocktype_write);
 }
 
-#ifdef BIND9
 /*
  * Create a writeable DLZ zone. This can be called by DLZ drivers
  * during configure() to create a zone that can be updated. The zone
@@ -528,23 +491,21 @@ dns_dlzunregister(dns_dlzimplementation_t **dlzimp) {
  * specific functionality on the zone
  */
 isc_result_t
-dns_dlz_writeablezone(dns_view_t *view, const char *zone_name) {
+dns_dlz_writeablezone(dns_view_t *view, dns_dlzdb_t *dlzdb,
+		      const char *zone_name)
+{
 	dns_zone_t *zone = NULL;
 	dns_zone_t *dupzone = NULL;
 	isc_result_t result;
 	isc_buffer_t buffer;
 	dns_fixedname_t fixorigin;
 	dns_name_t *origin;
-	dns_rdataclass_t zclass;
-	dns_dlzdb_t *dlzdatabase;
 
-	REQUIRE(DNS_DLZ_VALID(view->dlzdatabase));
+	REQUIRE(DNS_DLZ_VALID(dlzdb));
 
-	dlzdatabase = view->dlzdatabase;
+	REQUIRE(dlzdb->configure_callback != NULL);
 
-	REQUIRE(dlzdatabase->configure_callback != NULL);
-
-	isc_buffer_init(&buffer, zone_name, strlen(zone_name));
+	isc_buffer_constinit(&buffer, zone_name, strlen(zone_name));
 	isc_buffer_add(&buffer, strlen(zone_name));
 	dns_fixedname_init(&fixorigin);
 	result = dns_name_fromtext(dns_fixedname_name(&fixorigin),
@@ -553,7 +514,15 @@ dns_dlz_writeablezone(dns_view_t *view, const char *zone_name) {
 		goto cleanup;
 	origin = dns_fixedname_name(&fixorigin);
 
-	zclass = view->rdclass;
+	if (!dlzdb->search) {
+		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
+			      DNS_LOGMODULE_DLZ, ISC_LOG_WARNING,
+			      "DLZ %s has 'search no;', but attempted to "
+			      "register writeable zone %s.",
+			      dlzdb->dlzname, zone_name);
+		result = ISC_R_SUCCESS;
+		goto cleanup;
+	}
 
 	/* See if the zone already exists */
 	result = dns_view_findzone(view, origin, &dupzone);
@@ -575,25 +544,20 @@ dns_dlz_writeablezone(dns_view_t *view, const char *zone_name) {
 
 	dns_zone_setadded(zone, ISC_TRUE);
 
-	if (dlzdatabase->ssutable == NULL) {
-		result = dns_ssutable_createdlz(dlzdatabase->mctx,
-						&dlzdatabase->ssutable,
-						view->dlzdatabase);
+	if (dlzdb->ssutable == NULL) {
+		result = dns_ssutable_createdlz(dlzdb->mctx,
+						&dlzdb->ssutable, dlzdb);
 		if (result != ISC_R_SUCCESS)
 			goto cleanup;
 	}
-	dns_zone_setssutable(zone, dlzdatabase->ssutable);
+	dns_zone_setssutable(zone, dlzdb->ssutable);
 
-	result = dlzdatabase->configure_callback(view, zone);
+	result = dlzdb->configure_callback(view, dlzdb, zone);
 	if (result != ISC_R_SUCCESS)
 		goto cleanup;
 
-	/*
-	 * Add the zone to its view in the new view list.
-	 */
 	result = dns_view_addzone(view, zone);
 
-	result = ISC_R_SUCCESS;
 
  cleanup:
 	if (zone != NULL)
@@ -601,41 +565,37 @@ dns_dlz_writeablezone(dns_view_t *view, const char *zone_name) {
 
 	return (result);
 }
-#endif
 
 /*%
  * Configure a DLZ driver. This is optional, and if supplied gives
  * the backend an opportunity to configure parameters related to DLZ.
  */
 isc_result_t
-dns_dlzconfigure(dns_view_t *view, isc_result_t (*callback)(dns_view_t *,
-		 dns_zone_t *))
+dns_dlzconfigure(dns_view_t *view, dns_dlzdb_t *dlzdb,
+		 dlzconfigure_callback_t callback)
 {
 	dns_dlzimplementation_t *impl;
-	dns_dlzdb_t *dlzdatabase;
 	isc_result_t result;
 
-	REQUIRE(view != NULL);
-	REQUIRE(DNS_DLZ_VALID(view->dlzdatabase));
-	REQUIRE(view->dlzdatabase->implementation != NULL);
+	REQUIRE(DNS_DLZ_VALID(dlzdb));
+	REQUIRE(dlzdb->implementation != NULL);
 
-	dlzdatabase = view->dlzdatabase;
-	impl = dlzdatabase->implementation;
+	impl = dlzdb->implementation;
 
 	if (impl->methods->configure == NULL)
 		return (ISC_R_SUCCESS);
 
-	dlzdatabase->configure_callback = callback;
+	dlzdb->configure_callback = callback;
 
-	result = impl->methods->configure(impl->driverarg,
-					  dlzdatabase->dbdata, view);
+	result = impl->methods->configure(impl->driverarg, dlzdb->dbdata,
+					  view, dlzdb);
 	return (result);
 }
 
 isc_boolean_t
-dns_dlz_ssumatch(dns_dlzdb_t *dlzdatabase,
-		  dns_name_t *signer, dns_name_t *name, isc_netaddr_t *tcpaddr,
-		  dns_rdatatype_t type, const dst_key_t *key)
+dns_dlz_ssumatch(dns_dlzdb_t *dlzdatabase, dns_name_t *signer,
+		 dns_name_t *name, isc_netaddr_t *tcpaddr,
+		 dns_rdatatype_t type, const dst_key_t *key)
 {
 	dns_dlzimplementation_t *impl;
 	isc_boolean_t r;
