@@ -1,5 +1,6 @@
 /*
- * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 2004-2008, 2011, 2013, 2014  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 1999-2003  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -205,11 +206,6 @@ struct dns_adbfind {
  *      Must set _WANTEVENT for this to be meaningful.
  */
 #define DNS_ADBFIND_LAMEPRUNED		0x00000200
-/*%
- *      The server's fetch quota is exceeded; it will be treated as
- *      lame for this query.
- */
-#define DNS_ADBFIND_OVERQUOTA		0x00000400
 
 /*%
  * The answers to queries come back as a list of these.
@@ -218,7 +214,9 @@ struct dns_adbaddrinfo {
 	unsigned int			magic;		/*%< private */
 
 	isc_sockaddr_t			sockaddr;	/*%< [rw] */
-	unsigned int			srtt;		/*%< [rw] microseconds */
+	unsigned int			srtt;		/*%< [rw] microsecs */
+	isc_dscp_t			dscp;
+
 	unsigned int			flags;		/*%< [rw] */
 	dns_adbentry_t		       *entry;		/*%< private */
 	ISC_LINK(dns_adbaddrinfo_t)	publink;
@@ -338,13 +336,6 @@ dns_adb_createfind(dns_adb_t *adb, isc_task_t *task, isc_taskaction_t action,
 		   dns_rdatatype_t qtype, unsigned int options,
 		   isc_stdtime_t now, dns_name_t *target,
 		   in_port_t port, dns_adbfind_t **find);
-isc_result_t
-dns_adb_createfind2(dns_adb_t *adb, isc_task_t *task, isc_taskaction_t action,
-		    void *arg, dns_name_t *name, dns_name_t *qname,
-		    dns_rdatatype_t qtype, unsigned int options,
-		    isc_stdtime_t now, dns_name_t *target, in_port_t port,
-		    unsigned int depth, isc_counter_t *qc,
-		    dns_adbfind_t **find);
 /*%<
  * Main interface for clients. The adb will look up the name given in
  * "name" and will build up a list of found addresses, and perhaps start
@@ -521,12 +512,7 @@ dns_adb_marklame(dns_adb_t *adb, dns_adbaddrinfo_t *addr, dns_name_t *qname,
  */
 
 /*
- * Reasonable defaults for RTT adjustments
- *
- * (Note: these values function both as scaling factors and as
- * indicators of the type of RTT adjustment operation taking place.
- * Adjusting the scaling factors is fine, as long as they all remain
- * unique values.)
+ * A reasonable default for RTT adjustments
  */
 #define DNS_ADB_RTTADJDEFAULT		7	/*%< default scale */
 #define DNS_ADB_RTTADJREPLACE		0	/*%< replace with our rtt */
@@ -537,6 +523,18 @@ dns_adb_adjustsrtt(dns_adb_t *adb, dns_adbaddrinfo_t *addr,
 		   unsigned int rtt, unsigned int factor);
 /*%<
  * Mix the round trip time into the existing smoothed rtt.
+
+ * The formula used
+ * (where srtt is the existing rtt value, and rtt and factor are arguments to
+ * this function):
+ *
+ *\code
+ *	new_srtt = (old_srtt / 10 * factor) + (rtt / 10 * (10 - factor));
+ *\endcode
+ *
+ * XXXRTH  Do we want to publish the formula?  What if we want to change how
+ *         this works later on?  Recommend/require that the units are
+ *	   microseconds?
  *
  * Requires:
  *
@@ -552,24 +550,6 @@ dns_adb_adjustsrtt(dns_adb_t *adb, dns_adbaddrinfo_t *addr,
  *	srtt value.  This may include changes made by others.
  */
 
-void
-dns_adb_agesrtt(dns_adb_t *adb, dns_adbaddrinfo_t *addr, isc_stdtime_t now);
-/*
- * dns_adb_agesrtt is equivalent to dns_adb_adjustsrtt with factor
- * equal to DNS_ADB_RTTADJAGE and the current time passed in.
- *
- * Requires:
- *
- *\li	adb be valid.
- *
- *\li	addr be valid.
- *
- * Note:
- *
- *\li	The srtt in addr will be updated to reflect the new global
- *	srtt value.  This may include changes made by others.
- */
-
 void
 dns_adb_changeflags(dns_adb_t *adb, dns_adbaddrinfo_t *addr,
 		    unsigned int bits, unsigned int mask);
@@ -588,29 +568,92 @@ dns_adb_changeflags(dns_adb_t *adb, dns_adbaddrinfo_t *addr,
  */
 
 void
-dns_adb_plainresponse(dns_adb_t *adb, dns_adbaddrinfo_t *addr);
+dns_adb_setudpsize(dns_adb_t *adb, dns_adbaddrinfo_t *addr, unsigned int size);
 /*%
- * Record a successful DNS response.
+ * Update seen UDP response size.  The largest seen will be returned by
+ * dns_adb_getudpsize().
  *
  * Requires:
  *
- *\li  adb be valid.
+ *\li	adb be valid.
  *
- *\li  addr be valid.
+ *\li	addr be valid.
+ */
+
+unsigned int
+dns_adb_getudpsize(dns_adb_t *adb, dns_adbaddrinfo_t *addr);
+/*%
+ * Return the largest seen UDP response size.
+ *
+ * Requires:
+ *
+ *\li	adb be valid.
+ *
+ *\li	addr be valid.
+ */
+
+unsigned int
+dns_adb_probesize(dns_adb_t *adb, dns_adbaddrinfo_t *addr);
+/*%
+ * Return suggested EDNS UDP size based on observed responses / failures.
+ *
+ * Requires:
+ *
+ *\li	adb be valid.
+ *
+ *\li	addr be valid.
+ */
+
+void
+dns_adb_plainresponse(dns_adb_t *adb, dns_adbaddrinfo_t *addr);
+/*%
+ * Record a successful plain DNS response.
+ *
+ * Requires:
+ *
+ *\li	adb be valid.
+ *
+ *\li	addr be valid.
  */
 
 void
 dns_adb_timeout(dns_adb_t *adb, dns_adbaddrinfo_t *addr);
 /*%
- * Record a DNS UDP query failed.
+ * Record a plain DNS UDP query failed.
  *
  * Requires:
  *
- *\li  adb be valid.
+ *\li	adb be valid.
  *
- *\li  addr be valid.
+ *\li	addr be valid.
  */
 
+void
+dns_adb_ednsto(dns_adb_t *adb, dns_adbaddrinfo_t *addr, unsigned int size);
+/*%
+ * Record a failed EDNS UDP response and the advertised EDNS UDP buffer size
+ * used.
+ *
+ * Requires:
+ *
+ *\li	adb be valid.
+ *
+ *\li	addr be valid.
+ */
+
+isc_boolean_t
+dns_adb_noedns(dns_adb_t *adb, dns_adbaddrinfo_t *addr);
+/*%
+ * Return whether EDNS should be disabled for this server.
+ *
+ * Requires:
+ *
+ *\li	adb be valid.
+ *
+ *\li	addr be valid.
+ */
+
+
 isc_result_t
 dns_adb_findaddrinfo(dns_adb_t *adb, isc_sockaddr_t *sa,
 		     dns_adbaddrinfo_t **addrp, isc_stdtime_t now);
@@ -676,61 +719,40 @@ dns_adb_flushname(dns_adb_t *adb, dns_name_t *name);
  */
 
 void
-dns_adb_setquota(dns_adb_t *adb, isc_uint32_t quota, isc_uint32_t freq,
-		 double low, double high, double discount);
+dns_adb_flushnames(dns_adb_t *adb, dns_name_t *name);
 /*%<
- * Set the baseline ADB quota, and configure parameters for the
- * quota adjustment algorithm.
- *
- * If the number of fetches currently waiting for responses from this
- * address exceeds the current quota, then additional fetches are spilled.
- *
- * 'quota' is the highest permissible quota; it will adjust itself
- * downward in response to detected congestion.
- *
- * After every 'freq' fetches have either completed or timed out, an
- * exponentially weighted moving average of the ratio of timeouts
- * to responses is calculated.  If the EWMA goes above a 'high'
- * threshold, then the quota is adjusted down one step; if it drops
- * below a 'low' threshold, then the quota is adjusted back up one
- * step.
- *
- * The quota adjustment is based on the function (1 / 1 + (n/10)^(3/2)),
- * for values of n from 0 to 99.  It starts at 100% of the baseline
- * quota, and descends after 100 steps to 2%.
- *
- * 'discount' represents the discount rate of the moving average. Higher
- * values cause older values to be discounted sooner, providing a faster
- * response to changes in the timeout ratio.
+ * Flush 'name' and all subdomains from the adb cache.
  *
  * Requires:
  *\li	'adb' is valid.
+ *\li	'name' is valid.
  */
 
-isc_boolean_t
-dns_adbentry_overquota(dns_adbentry_t *entry);
+void
+dns_adb_setsit(dns_adb_t *adb, dns_adbaddrinfo_t *addr,
+	       const unsigned char *sit, size_t len);
 /*%<
- * Returns true if the specified ADB has too many active fetches.
+ * Record the Source Identity Token (SIT) associated with this addresss.  If
+ * sit is NULL or len is zero. The recorded SIT is cleared.
  *
  * Requires:
- *\li	'entry' is valid.
+ *\li	'adb' is valid.
+ *\li	'addr' is valid.
  */
 
-void
-dns_adb_beginudpfetch(dns_adb_t *adb, dns_adbaddrinfo_t *addr);
-void
-dns_adb_endudpfetch(dns_adb_t *adb, dns_adbaddrinfo_t *addr);
-/*%
- * Begin/end a UDP fetch on a particular address.
- *
- * These functions increment or decrement the fetch counter for
- * the ADB entry so that the fetch quota can be enforced.
+size_t
+dns_adb_getsit(dns_adb_t *adb, dns_adbaddrinfo_t *addr,
+	       unsigned char *sit, size_t len);
+/*
+ * Retieve the saved SIT value and store it in 'sit' which has size 'len'.
  *
  * Requires:
+ *\li	'adb' is valid.
+ *\li	'addr' is valid.
  *
- *\li	adb be valid.
- *
- *\li	addr be valid.
+ * Returns:
+ *	The size of the sit token or zero if it doesn't fit in the buffer
+ *	or it doesn't exist.
  */
 
 ISC_LANG_ENDDECLS
