@@ -1,5 +1,6 @@
 /*
- * Portions Copyright (C) Internet Systems Consortium, Inc. ("ISC")
+ * Portions Copyright (C) 2004-2009, 2011-2014  Internet Systems Consortium, Inc. ("ISC")
+ * Portions Copyright (C) 1999-2002  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -13,10 +14,7 @@
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
  * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * See the COPYRIGHT file distributed with this work for additional
- * information regarding copyright ownership.
- *
- * Portions Copyright (C) Network Associates, Inc.
+ * Portions Copyright (C) 1995-2000 by Network Associates, Inc.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -31,6 +29,8 @@
  * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+/* $Id$ */
+
 #ifdef OPENSSL
 #ifndef USE_EVP
 #define USE_EVP 1
@@ -42,7 +42,6 @@
 
 #include <isc/entropy.h>
 #include <isc/mem.h>
-#include <isc/safe.h>
 #include <isc/sha1.h>
 #include <isc/util.h>
 
@@ -56,79 +55,6 @@
 
 static isc_result_t openssldsa_todns(const dst_key_t *key, isc_buffer_t *data);
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
-static void
-DSA_get0_pqg(const DSA *d, const BIGNUM **p, const BIGNUM **q,
-	     const BIGNUM **g)
-{
-	if (p != NULL)
-		*p = d->p;
-	if (q != NULL)
-		*q = d->q;
-	if (g != NULL)
-		*g = d->g;
-}
-
-static int
-DSA_set0_pqg(DSA *d, BIGNUM *p, BIGNUM *q, BIGNUM *g) {
-	if (p == NULL || q == NULL || g == NULL)
-		return 0;
-	BN_free(d->p);
-	BN_free(d->q);
-	BN_free(d->g);
-	d->p = p;
-	d->q = q;
-	d->g = g;
-
-	return 1;
-}
-
-static void
-DSA_get0_key(const DSA *d, const BIGNUM **pub_key, const BIGNUM **priv_key) {
-	if (pub_key != NULL)
-		*pub_key = d->pub_key;
-	if (priv_key != NULL)
-		*priv_key = d->priv_key;
-}
-
-static int
-DSA_set0_key(DSA *d, BIGNUM *pub_key, BIGNUM *priv_key) {
-	/* Note that it is valid for priv_key to be NULL */
-	if (pub_key == NULL)
-		return 0;
-
-	BN_free(d->pub_key);
-	BN_free(d->priv_key);
-	d->pub_key = pub_key;
-	d->priv_key = priv_key;
-
-	return 1;
-}
-
-static void
-DSA_SIG_get0(const DSA_SIG *sig, const BIGNUM **pr, const BIGNUM **ps) {
-	*pr = sig->r;
-	*ps = sig->s;
-}
-
-static int
-DSA_SIG_set0(DSA_SIG *sig, BIGNUM *r, BIGNUM *s) {
-	if (r == NULL || s == NULL)
-		return 0;
-
-	BN_clear_free(sig->r);
-	BN_clear_free(sig->s);
-	sig->r = r;
-	sig->s = s;
-
-	return 1;
-}
-
-
-#define DSA_clear_flags(d, x) (d)->flags &= ~(x)
-
-#endif
-
 static isc_result_t
 openssldsa_createctx(dst_key_t *key, dst_context_t *dctx) {
 #if USE_EVP
@@ -154,8 +80,6 @@ openssldsa_createctx(dst_key_t *key, dst_context_t *dctx) {
 	UNUSED(key);
 
 	sha1ctx = isc_mem_get(dctx->mctx, sizeof(isc_sha1_t));
-	if (sha1ctx == NULL)
-		return (ISC_R_NOMEMORY);
 	isc_sha1_init(sha1ctx);
 	dctx->ctxdata.sha1ctx = sha1ctx;
 	return (ISC_R_SUCCESS);
@@ -199,7 +123,7 @@ openssldsa_adddata(dst_context_t *dctx, const isc_region_t *data) {
 }
 
 static int
-BN_bn2bin_fixed(const BIGNUM *bn, unsigned char *buf, int size) {
+BN_bn2bin_fixed(BIGNUM *bn, unsigned char *buf, int size) {
 	int bytes = size - BN_num_bytes(bn);
 	while (bytes-- > 0)
 		*buf++ = 0;
@@ -211,10 +135,8 @@ static isc_result_t
 openssldsa_sign(dst_context_t *dctx, isc_buffer_t *sig) {
 	dst_key_t *key = dctx->key;
 	DSA *dsa = key->keydata.dsa;
-	isc_region_t region;
+	isc_region_t r;
 	DSA_SIG *dsasig;
-	const BIGNUM *r = 0, *s = NULL;
-	unsigned int klen;
 #if USE_EVP
 	EVP_MD_CTX *evp_md_ctx = dctx->ctxdata.evp_md_ctx;
 	EVP_PKEY *pkey;
@@ -226,8 +148,8 @@ openssldsa_sign(dst_context_t *dctx, isc_buffer_t *sig) {
 	unsigned char digest[ISC_SHA1_DIGESTLENGTH];
 #endif
 
-	isc_buffer_availableregion(sig, &region);
-	if (region.length < ISC_SHA1_DIGESTLENGTH * 2 + 1)
+	isc_buffer_availableregion(sig, &r);
+	if (r.length < ISC_SHA1_DIGESTLENGTH * 2 + 1)
 		return (ISC_R_NOSPACE);
 
 #if USE_EVP
@@ -266,7 +188,6 @@ openssldsa_sign(dst_context_t *dctx, isc_buffer_t *sig) {
 					       ISC_R_FAILURE));
 	}
 	free(sigbuf);
-
 #elif 0
 	/* Only use EVP for the Digest */
 	if (!EVP_DigestFinal_ex(evp_md_ctx, digest, &siglen)) {
@@ -288,18 +209,11 @@ openssldsa_sign(dst_context_t *dctx, isc_buffer_t *sig) {
 					       "DSA_do_sign",
 					       DST_R_SIGNFAILURE));
 #endif
-
-	klen = (key->key_size - 512)/64;
-	if (klen > 255)
-		return (ISC_R_FAILURE);
-	*region.base = klen;
-	isc_region_consume(&region, 1);
-
-	DSA_SIG_get0(dsasig, &r, &s);
-	BN_bn2bin_fixed(r, region.base, ISC_SHA1_DIGESTLENGTH);
-	isc_region_consume(&region, ISC_SHA1_DIGESTLENGTH);
-	BN_bn2bin_fixed(s, region.base, ISC_SHA1_DIGESTLENGTH);
-	isc_region_consume(&region, ISC_SHA1_DIGESTLENGTH);
+	*r.base++ = (key->key_size - 512)/64;
+	BN_bn2bin_fixed(dsasig->r, r.base, ISC_SHA1_DIGESTLENGTH);
+	r.base += ISC_SHA1_DIGESTLENGTH;
+	BN_bn2bin_fixed(dsasig->s, r.base, ISC_SHA1_DIGESTLENGTH);
+	r.base += ISC_SHA1_DIGESTLENGTH;
 	DSA_SIG_free(dsasig);
 	isc_buffer_add(sig, ISC_SHA1_DIGESTLENGTH * 2 + 1);
 
@@ -310,7 +224,6 @@ static isc_result_t
 openssldsa_verify(dst_context_t *dctx, const isc_region_t *sig) {
 	dst_key_t *key = dctx->key;
 	DSA *dsa = key->keydata.dsa;
-	BIGNUM *r = NULL, *s = NULL;
 	int status = 0;
 	unsigned char *cp = sig->base;
 	DSA_SIG *dsasig;
@@ -346,10 +259,9 @@ openssldsa_verify(dst_context_t *dctx, const isc_region_t *sig) {
 	dsasig = DSA_SIG_new();
 	if (dsasig == NULL)
 		return (ISC_R_NOMEMORY);
-	r = BN_bin2bn(cp, ISC_SHA1_DIGESTLENGTH, NULL);
+	dsasig->r = BN_bin2bn(cp, ISC_SHA1_DIGESTLENGTH, NULL);
 	cp += ISC_SHA1_DIGESTLENGTH;
-	s = BN_bin2bn(cp, ISC_SHA1_DIGESTLENGTH, NULL);
-	DSA_SIG_set0(dsasig, r, s);
+	dsasig->s = BN_bin2bn(cp, ISC_SHA1_DIGESTLENGTH, NULL);
 
 #if 0
 	pkey = EVP_PKEY_new();
@@ -388,11 +300,8 @@ openssldsa_verify(dst_context_t *dctx, const isc_region_t *sig) {
 
 static isc_boolean_t
 openssldsa_compare(const dst_key_t *key1, const dst_key_t *key2) {
+	int status;
 	DSA *dsa1, *dsa2;
-	const BIGNUM *pub_key1 = NULL, *priv_key1 = NULL;
-	const BIGNUM *pub_key2 = NULL, *priv_key2 = NULL;
-	const BIGNUM *p1 = NULL, *q1 = NULL, *g1 = NULL;
-	const BIGNUM *p2 = NULL, *q2 = NULL, *g2 = NULL;
 
 	dsa1 = key1->keydata.dsa;
 	dsa2 = key2->keydata.dsa;
@@ -402,19 +311,18 @@ openssldsa_compare(const dst_key_t *key1, const dst_key_t *key2) {
 	else if (dsa1 == NULL || dsa2 == NULL)
 		return (ISC_FALSE);
 
-	DSA_get0_key(dsa1, &pub_key1, &priv_key1);
-	DSA_get0_key(dsa2, &pub_key2, &priv_key2);
-	DSA_get0_pqg(dsa1, &p1, &q1, &g1);
-	DSA_get0_pqg(dsa2, &p2, &q2, &g2);
+	status = BN_cmp(dsa1->p, dsa2->p) ||
+		 BN_cmp(dsa1->q, dsa2->q) ||
+		 BN_cmp(dsa1->g, dsa2->g) ||
+		 BN_cmp(dsa1->pub_key, dsa2->pub_key);
 
-	if (BN_cmp(p1, p2) != 0 || BN_cmp(q1, q2) != 0 ||
-	    BN_cmp(g1, g2) != 0 || BN_cmp(pub_key1, pub_key2) != 0)
+	if (status != 0)
 		return (ISC_FALSE);
 
-	if (priv_key1 != NULL || priv_key2 != NULL) {
-		if (priv_key1 == NULL || priv_key2 == NULL)
+	if (dsa1->priv_key != NULL || dsa2->priv_key != NULL) {
+		if (dsa1->priv_key == NULL || dsa2->priv_key == NULL)
 			return (ISC_FALSE);
-		if (BN_cmp(priv_key1, priv_key2))
+		if (BN_cmp(dsa1->priv_key, dsa2->priv_key))
 			return (ISC_FALSE);
 	}
 	return (ISC_TRUE);
@@ -422,7 +330,8 @@ openssldsa_compare(const dst_key_t *key1, const dst_key_t *key2) {
 
 #if OPENSSL_VERSION_NUMBER > 0x00908000L
 static int
-progress_cb(int p, int n, BN_GENCB *cb) {
+progress_cb(int p, int n, BN_GENCB *cb)
+{
 	union {
 		void *dptr;
 		void (*fptr)(int);
@@ -430,7 +339,7 @@ progress_cb(int p, int n, BN_GENCB *cb) {
 
 	UNUSED(n);
 
-	u.dptr = BN_GENCB_get_arg(cb);
+	u.dptr = cb->arg;
 	if (u.fptr != NULL)
 		u.fptr(p);
 	return (1);
@@ -443,10 +352,7 @@ openssldsa_generate(dst_key_t *key, int unused, void (*callback)(int)) {
 	unsigned char rand_array[ISC_SHA1_DIGESTLENGTH];
 	isc_result_t result;
 #if OPENSSL_VERSION_NUMBER > 0x00908000L
-	BN_GENCB *cb;
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
-	BN_GENCB _cb;
-#endif
+	BN_GENCB cb;
 	union {
 		void *dptr;
 		void (*fptr)(int);
@@ -467,31 +373,22 @@ openssldsa_generate(dst_key_t *key, int unused, void (*callback)(int)) {
 	dsa = DSA_new();
 	if (dsa == NULL)
 		return (dst__openssl_toresult(DST_R_OPENSSLFAILURE));
-	cb = BN_GENCB_new();
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
-	if (cb == NULL) {
-		DSA_free(dsa);
-		return (dst__openssl_toresult(DST_R_OPENSSLFAILURE));
-	}
-#endif
+
 	if (callback == NULL) {
-		BN_GENCB_set_old(cb, NULL, NULL);
+		BN_GENCB_set_old(&cb, NULL, NULL);
 	} else {
 		u.fptr = callback;
-		BN_GENCB_set(cb, &progress_cb, u.dptr);
+		BN_GENCB_set(&cb, &progress_cb, u.dptr);
 	}
 
 	if (!DSA_generate_parameters_ex(dsa, key->key_size, rand_array,
 					ISC_SHA1_DIGESTLENGTH,  NULL, NULL,
-					cb))
+					&cb))
 	{
 		DSA_free(dsa);
-		BN_GENCB_free(cb);
 		return (dst__openssl_toresult2("DSA_generate_parameters_ex",
 					       DST_R_OPENSSLFAILURE));
 	}
-	BN_GENCB_free(cb);
-	cb = NULL;
 #else
 	dsa = DSA_generate_parameters(key->key_size, rand_array,
 				      ISC_SHA1_DIGESTLENGTH, NULL, NULL,
@@ -506,8 +403,7 @@ openssldsa_generate(dst_key_t *key, int unused, void (*callback)(int)) {
 		return (dst__openssl_toresult2("DSA_generate_key",
 					       DST_R_OPENSSLFAILURE));
 	}
-
-	DSA_clear_flags(dsa, DSA_FLAG_CACHE_MONT_P);
+	dsa->flags &= ~DSA_FLAG_CACHE_MONT_P;
 
 	key->keydata.dsa = dsa;
 
@@ -517,10 +413,7 @@ openssldsa_generate(dst_key_t *key, int unused, void (*callback)(int)) {
 static isc_boolean_t
 openssldsa_isprivate(const dst_key_t *key) {
 	DSA *dsa = key->keydata.dsa;
-	const BIGNUM *priv_key = NULL;
-
-	DSA_get0_key(dsa, NULL, &priv_key);
-	return (ISC_TF(dsa != NULL && priv_key != NULL));
+	return (ISC_TF(dsa != NULL && dsa->priv_key != NULL));
 }
 
 static void
@@ -534,7 +427,6 @@ openssldsa_destroy(dst_key_t *key) {
 static isc_result_t
 openssldsa_todns(const dst_key_t *key, isc_buffer_t *data) {
 	DSA *dsa;
-	const BIGNUM *pub_key, *p = NULL, *q = NULL, *g = NULL;
 	isc_region_t r;
 	int dnslen;
 	unsigned int t, p_bytes;
@@ -545,10 +437,7 @@ openssldsa_todns(const dst_key_t *key, isc_buffer_t *data) {
 
 	isc_buffer_availableregion(data, &r);
 
-	DSA_get0_key(dsa, &pub_key, NULL);
-	DSA_get0_pqg(dsa, &p, &q, &g);
-
-	t = (BN_num_bytes(p) - 64) / 8;
+	t = (BN_num_bytes(dsa->p) - 64) / 8;
 	if (t > 8)
 		return (DST_R_INVALIDPUBLICKEY);
 	p_bytes = 64 + 8 * t;
@@ -557,17 +446,15 @@ openssldsa_todns(const dst_key_t *key, isc_buffer_t *data) {
 	if (r.length < (unsigned int) dnslen)
 		return (ISC_R_NOSPACE);
 
-	*r.base = t;
-	isc_region_consume(&r, 1);
-
-	BN_bn2bin_fixed(q, r.base, ISC_SHA1_DIGESTLENGTH);
-	isc_region_consume(&r, ISC_SHA1_DIGESTLENGTH);
-	BN_bn2bin_fixed(p, r.base, key->key_size/8);
-	isc_region_consume(&r, p_bytes);
-	BN_bn2bin_fixed(g, r.base, key->key_size/8);
-	isc_region_consume(&r, p_bytes);
-	BN_bn2bin_fixed(pub_key, r.base, key->key_size/8);
-	isc_region_consume(&r, p_bytes);
+	*r.base++ = t;
+	BN_bn2bin_fixed(dsa->q, r.base, ISC_SHA1_DIGESTLENGTH);
+	r.base += ISC_SHA1_DIGESTLENGTH;
+	BN_bn2bin_fixed(dsa->p, r.base, key->key_size/8);
+	r.base += p_bytes;
+	BN_bn2bin_fixed(dsa->g, r.base, key->key_size/8);
+	r.base += p_bytes;
+	BN_bn2bin_fixed(dsa->pub_key, r.base, key->key_size/8);
+	r.base += p_bytes;
 
 	isc_buffer_add(data, dnslen);
 
@@ -577,7 +464,6 @@ openssldsa_todns(const dst_key_t *key, isc_buffer_t *data) {
 static isc_result_t
 openssldsa_fromdns(dst_key_t *key, isc_buffer_t *data) {
 	DSA *dsa;
-	BIGNUM *pub_key, *p, *q, *g;
 	isc_region_t r;
 	unsigned int t, p_bytes;
 	isc_mem_t *mctx = key->mctx;
@@ -591,43 +477,31 @@ openssldsa_fromdns(dst_key_t *key, isc_buffer_t *data) {
 	dsa = DSA_new();
 	if (dsa == NULL)
 		return (ISC_R_NOMEMORY);
-	DSA_clear_flags(dsa, DSA_FLAG_CACHE_MONT_P);
+	dsa->flags &= ~DSA_FLAG_CACHE_MONT_P;
 
-	t = (unsigned int) *r.base;
-	isc_region_consume(&r, 1);
+	t = (unsigned int) *r.base++;
 	if (t > 8) {
 		DSA_free(dsa);
 		return (DST_R_INVALIDPUBLICKEY);
 	}
 	p_bytes = 64 + 8 * t;
 
-	if (r.length < ISC_SHA1_DIGESTLENGTH + 3 * p_bytes) {
+	if (r.length < 1 + ISC_SHA1_DIGESTLENGTH + 3 * p_bytes) {
 		DSA_free(dsa);
 		return (DST_R_INVALIDPUBLICKEY);
 	}
 
-	q = BN_bin2bn(r.base, ISC_SHA1_DIGESTLENGTH, NULL);
-	isc_region_consume(&r, ISC_SHA1_DIGESTLENGTH);
+	dsa->q = BN_bin2bn(r.base, ISC_SHA1_DIGESTLENGTH, NULL);
+	r.base += ISC_SHA1_DIGESTLENGTH;
 
-	p = BN_bin2bn(r.base, p_bytes, NULL);
-	isc_region_consume(&r, p_bytes);
+	dsa->p = BN_bin2bn(r.base, p_bytes, NULL);
+	r.base += p_bytes;
 
-	g = BN_bin2bn(r.base, p_bytes, NULL);
-	isc_region_consume(&r, p_bytes);
+	dsa->g = BN_bin2bn(r.base, p_bytes, NULL);
+	r.base += p_bytes;
 
-	pub_key = BN_bin2bn(r.base, p_bytes, NULL);
-	isc_region_consume(&r, p_bytes);
-
-	if (pub_key == NULL || p == NULL || q == NULL || g == NULL) {
-		DSA_free(dsa);
-		if (p != NULL) BN_free(p);
-		if (q != NULL) BN_free(q);
-		if (g != NULL) BN_free(g);
-		return (ISC_R_NOMEMORY);
-	}
-
-	DSA_set0_key(dsa, pub_key, NULL);
-	DSA_set0_pqg(dsa, p, q, g);
+	dsa->pub_key = BN_bin2bn(r.base, p_bytes, NULL);
+	r.base += p_bytes;
 
 	key->key_size = p_bytes * 8;
 
@@ -643,8 +517,6 @@ static isc_result_t
 openssldsa_tofile(const dst_key_t *key, const char *directory) {
 	int cnt = 0;
 	DSA *dsa;
-	const BIGNUM *pub_key = NULL, *priv_key = NULL;
-	const BIGNUM *p = NULL, *q = NULL, *g = NULL;
 	dst_private_t priv;
 	unsigned char bufs[5][128];
 
@@ -658,36 +530,33 @@ openssldsa_tofile(const dst_key_t *key, const char *directory) {
 
 	dsa = key->keydata.dsa;
 
-	DSA_get0_key(dsa, &pub_key, &priv_key);
-	DSA_get0_pqg(dsa, &p, &q, &g);
-
 	priv.elements[cnt].tag = TAG_DSA_PRIME;
-	priv.elements[cnt].length = BN_num_bytes(p);
-	BN_bn2bin(p, bufs[cnt]);
+	priv.elements[cnt].length = BN_num_bytes(dsa->p);
+	BN_bn2bin(dsa->p, bufs[cnt]);
 	priv.elements[cnt].data = bufs[cnt];
 	cnt++;
 
 	priv.elements[cnt].tag = TAG_DSA_SUBPRIME;
-	priv.elements[cnt].length = BN_num_bytes(q);
-	BN_bn2bin(q, bufs[cnt]);
+	priv.elements[cnt].length = BN_num_bytes(dsa->q);
+	BN_bn2bin(dsa->q, bufs[cnt]);
 	priv.elements[cnt].data = bufs[cnt];
 	cnt++;
 
 	priv.elements[cnt].tag = TAG_DSA_BASE;
-	priv.elements[cnt].length = BN_num_bytes(g);
-	BN_bn2bin(g, bufs[cnt]);
+	priv.elements[cnt].length = BN_num_bytes(dsa->g);
+	BN_bn2bin(dsa->g, bufs[cnt]);
 	priv.elements[cnt].data = bufs[cnt];
 	cnt++;
 
 	priv.elements[cnt].tag = TAG_DSA_PRIVATE;
-	priv.elements[cnt].length = BN_num_bytes(priv_key);
-	BN_bn2bin(priv_key, bufs[cnt]);
+	priv.elements[cnt].length = BN_num_bytes(dsa->priv_key);
+	BN_bn2bin(dsa->priv_key, bufs[cnt]);
 	priv.elements[cnt].data = bufs[cnt];
 	cnt++;
 
 	priv.elements[cnt].tag = TAG_DSA_PUBLIC;
-	priv.elements[cnt].length = BN_num_bytes(pub_key);
-	BN_bn2bin(pub_key, bufs[cnt]);
+	priv.elements[cnt].length = BN_num_bytes(dsa->pub_key);
+	BN_bn2bin(dsa->pub_key, bufs[cnt]);
 	priv.elements[cnt].data = bufs[cnt];
 	cnt++;
 
@@ -701,8 +570,6 @@ openssldsa_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 	isc_result_t ret;
 	int i;
 	DSA *dsa = NULL;
-	BIGNUM *pub_key = NULL, *priv_key = NULL;
-	BIGNUM *p = NULL, *q = NULL, *g = NULL;
 	isc_mem_t *mctx = key->mctx;
 #define DST_RET(a) {ret = a; goto err;}
 
@@ -720,14 +587,14 @@ openssldsa_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 		pub->keydata.pkey = NULL;
 		key->key_size = pub->key_size;
 		dst__privstruct_free(&priv, mctx);
-		isc_safe_memwipe(&priv, sizeof(priv));
+		memset(&priv, 0, sizeof(priv));
 		return (ISC_R_SUCCESS);
 	}
 
 	dsa = DSA_new();
 	if (dsa == NULL)
 		DST_RET(ISC_R_NOMEMORY);
-	DSA_clear_flags(dsa, DSA_FLAG_CACHE_MONT_P);
+	dsa->flags &= ~DSA_FLAG_CACHE_MONT_P;
 	key->keydata.dsa = dsa;
 
 	for (i = 0; i < priv.nelements; i++) {
@@ -739,44 +606,37 @@ openssldsa_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 
 		switch (priv.elements[i].tag) {
 			case TAG_DSA_PRIME:
-				p = bn;
+				dsa->p = bn;
 				break;
 			case TAG_DSA_SUBPRIME:
-				q = bn;
+				dsa->q = bn;
 				break;
 			case TAG_DSA_BASE:
-				g = bn;
+				dsa->g = bn;
 				break;
 			case TAG_DSA_PRIVATE:
-				priv_key = bn;
+				dsa->priv_key = bn;
 				break;
 			case TAG_DSA_PUBLIC:
-				pub_key = bn;
+				dsa->pub_key = bn;
 				break;
 		}
 	}
 	dst__privstruct_free(&priv, mctx);
-	isc_safe_memwipe(&priv, sizeof(priv));
-	DSA_set0_key(dsa, pub_key, priv_key);
-	DSA_set0_pqg(dsa, p, q, g);
-	key->key_size = BN_num_bits(p);
+	memset(&priv, 0, sizeof(priv));
+	key->key_size = BN_num_bits(dsa->p);
 	return (ISC_R_SUCCESS);
 
  err:
-	if (p != NULL)
-		BN_free(p);
-	if (q != NULL)
-		BN_free(q);
-	if (g != NULL)
-		BN_free(g);
 	openssldsa_destroy(key);
 	dst__privstruct_free(&priv, mctx);
-	isc_safe_memwipe(&priv, sizeof(priv));
+	memset(&priv, 0, sizeof(priv));
 	return (ret);
 }
 
 static dst_func_t openssldsa_functions = {
 	openssldsa_createctx,
+	NULL, /*%< createctx2 */
 	openssldsa_destroyctx,
 	openssldsa_adddata,
 	openssldsa_sign,
